/* split_report.c -- splits up turtle stamp list into multiple files */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define BUFSIZE     1024
#define INDEX_FILE  "index.html"
#define IMAGE_DIR   "../images"
#define IMAGE_URL   "../../images"
#define NO_IMAGE    "invisible"
#define ALT         "sample stamp"
#define HOME        "http://www.personal.psu.edu/crr2/turtstmp"

void  htmlhead (FILE *fp, char *title);
int   getline  (char *buf);
void  weedout  (char *buf);
void  compact  (char *buf);
int   starts   (char *s, char *pat);
char *strtrim  (char *s);
char *usage    (void);
void  fatal    (char *msg);

int main (int argc, char **argv) {
	int  imageopt = 0;     /* pacify lint */
	char *dname = "";      /* pacify lint */
	char buf   [BUFSIZE];
	char iname [BUFSIZE];
	char fname [BUFSIZE];
	FILE *ix;
	FILE *fp = NULL;
	char ccode [BUFSIZE];
	char cname [BUFSIZE];
	char *image;
	char *p;
	int  letter = 'A' - 1;
	char cmdbuf [BUFSIZE];

	/* get arguments */
	if (argc == 2) {
		imageopt = 0;
		dname = argv[1];
	}
	else if (argc == 3 && !strcmp (argv[1], "-images")) {
		imageopt = 1;
		dname = argv[2];
	}
	else
		fatal (usage());

	/* check dirname */
	sprintf (cmdbuf, "test -d %s", dname);
	if (system (cmdbuf)) {     /* zero => true; non-zero => false */
		sprintf (cmdbuf, "no such directory: %s\n%s", dname, usage());
		fatal (cmdbuf);
	}

	/* start index file */
	sprintf (iname, "%s/%s", dname, INDEX_FILE);
//	fprintf (stderr, "opening %s\n", iname);
	if (!(ix = fopen (iname, "w")))
		fatal ("error opening output file");
	htmlhead (ix, "Country Index");
	fprintf (ix, "<br>\n");
	fprintf (ix, "<center><h2>COUNTRY INDEX</h2></center>\n");
	fprintf (ix, "<center><i>\n");
	fprintf (ix, "  <a href=\"#A\">A</a> <a href=\"#B\">B</a> <a href=\"#C\">C</a>\n");
	fprintf (ix, "  <a href=\"#D\">D</a> <a href=\"#E\">E</a> <a href=\"#F\">F</a>\n");
	fprintf (ix, "  <a href=\"#G\">G</a> <a href=\"#H\">H</a> <a href=\"#I\">I</a>\n");
	fprintf (ix, "  <a href=\"#J\">J</a> <a href=\"#K\">K</a> <a href=\"#L\">L</a>\n");
	fprintf (ix, "  <a href=\"#M\">M</a> <a href=\"#N\">N</a> <a href=\"#O\">O</a>\n");
	fprintf (ix, "  <a href=\"#P\">P</a> <a href=\"#Q\">Q</a> <a href=\"#R\">R</a>\n");
	fprintf (ix, "  <a href=\"#S\">S</a> <a href=\"#T\">T</a> <a href=\"#U\">U</a>\n");
	fprintf (ix, "  <a href=\"#V\">V</a> <a href=\"#W\">W</a> <a href=\"#X\">X</a>\n");
	fprintf (ix, "  <a href=\"#Y\">Y</a> <a href=\"#Z\">Z</a>\n");
	fprintf (ix, "</i></center>\n");
	fprintf (ix, "<p>\n");
	fprintf (ix, "<dl>\n");

	/* process input file */
	while (getchar() != '@')           /* skip preliminary printer codes */
		;
	ungetc ('@', stdin);
	while (getline (buf) != EOF) {     /* read a cleaned-up line into buf
	                                      (\n et al. replaced with \0) */
		if (starts (buf, "@@@")) {
			/* get country code and name */
			strcpy (ccode, strtrim (buf + 3));
			if ((p = strstr (ccode, ".html")))
				*p = '\0';
			if (getline (buf) == EOF)
				fatal ("unexpected EOF");
			strcpy (cname, strtrim (buf));

			/* close old file and open new one */
			if (fp)
				fclose (fp);
			sprintf (fname, "%s/%s.html", dname, ccode);
//			fprintf (stderr, "opening %s\n", fname);
			if (!(fp = fopen (fname, "w")))
				fatal ("error opening output file");

			/* add link to index */
			while (letter < *cname) {
				++letter;
				fprintf (ix, "<dt><br><a name=\"%c\"><big>-- <b>%c</b> --</big></a>\n",
				             letter, letter);
				fprintf (ix, "<dd>\n");
			}
			fprintf (ix, "\t<a href=\"%s.html\">", ccode);
			fprintf (ix, "%s</a><br>\n", cname);
		}
		else {
			if (!fp)
				fatal ("no open output file");
			if (starts (buf, "<html>")) {
				/* replace html header lines */
				while (getline (buf) != EOF && !starts (buf, "<body"))
					;
				if (!starts (buf, "<body"))
					fatal ("unexpected EOF");
				htmlhead (fp, cname);

				/* add image */
				if (imageopt) {
					sprintf (cmdbuf, "test -f %s/%s.jpg", IMAGE_DIR, ccode);
					image = (!system (cmdbuf) ? ccode : NO_IMAGE);
					fprintf (fp, "<table width=\"100%%\">\n");
					if (!strcmp (ccode, "SAR2")) {          /* special case for Sarangarh */
						fprintf (fp, "<tr>\n");
						fprintf (fp, "<td valign=top align=right>");
						fprintf (fp, "<img src=\"%s/SAR2_2.jpg\" border=0 alt=\"%s\"></td>\n",
						             IMAGE_URL, ALT);
						fprintf (fp, "<td valign=top align=right>");
						fprintf (fp, "<img src=\"%s/SAR2.jpg\" border=0 alt=\"%s\"><br>\n",
						             IMAGE_URL, ALT);
						fprintf (fp, "<font size=-1><br>");
						fprintf (fp, "(The image on the right is a colorized photocopy<br>\n");
						fprintf (fp, "to show the location of the tortoise.)</font></td>\n");
						fprintf (fp, "</tr>\n");
					}
					fprintf (fp, "<tr>\n");
					fprintf (fp, "<td valign=bottom>\n");
					while (getline (buf) != EOF && !starts (buf, "<p>"))
						fprintf (fp, "%s\n", buf);
					if (!starts (buf, "<p>"))
						fatal ("unexpected EOF");
					fprintf (fp, "</td>\n");
					if (!strcmp (ccode, "SAR2")) {          /* special case for Sarangarh */
						fprintf (fp, "<td align=right>&nbsp;</td>\n");     /* images above */
					}
					else if (!strcmp (ccode, "GER3")) {     /* special case for Germany */
						fprintf (fp, "<td align=right>");
						fprintf (fp, "<img src=\"%s/GER3.jpg\" border=0 alt=\"%s\"><br>\n",
						             IMAGE_URL, ALT);
						fprintf (fp, "<font size=-1><br>");
						fprintf (fp, "Mailing Label with Enlarged Inset<br>\n");
						fprintf (fp, "to Show Detail of Turtle</font></td>\n");
					}
					else {
						fprintf (fp, "<td align=right>");
						fprintf (fp, "<img src=\"%s/%s.jpg\" border=0 alt=\"%s\"></td>\n",
						             IMAGE_URL, image, ALT);
					}
					fprintf (fp, "</tr>\n");
					fprintf (fp, "</table>\n");
					fprintf (fp, "%s\n", buf);     /* <p> */
				}
			}
			else if (starts (buf, "<i>Last updated:")) {
				/* put home link on the left, and date on the right */
				fprintf (fp, "<table width=\"100%%\">\n");
				fprintf (fp, "<tr valign=baseline>\n");
				fprintf (fp, "<td align=left><a href=\"%s/\">Main page</a></td>\n", HOME);
				fprintf (fp, "<td align=right>%s</td>\n", buf);
				fprintf (fp, "</tr>\n");
				fprintf (fp, "</table>\n");
			}
			else {
				/* copy line as is */
				fprintf (fp, "%s\n", buf);
			}
		}
	}

	/* wrap up */
	if (fp)
		fclose (fp);
	while (letter < 'Z') {
		++letter;
		fprintf (ix, "<dt><br><a name=\"%c\"><big>-- <b>%c</b> --</big></a>\n",
		             letter, letter);
		fprintf (ix, "<dd>\n");
	}
	fprintf (ix, "</dl>\n");
	fprintf (ix, "<p>\n");
	fprintf (ix, "</body>\n");
	fprintf (ix, "</html>\n");
	fclose (ix);
	exit (0);
}

void htmlhead (FILE *fp, char *title) {
	fprintf (fp, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n");
	fprintf (fp, "\t\"http://www.w3.org/TR/html4/loose.dtd\">\n");
	fprintf (fp, "<html>\n");
	fprintf (fp, "<head>\n");
	fprintf (fp, "<title>%s</title>\n", title);
	fprintf (fp, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
	fprintf (fp, "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n");
	fprintf (fp, "<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/turtstmp.css\">\n", HOME);
	fprintf (fp, "<link rel=\"shortcut icon\" href=\"%s/turtstmp.ico\">\n", HOME);
	fprintf (fp, "</head>\n");
	fprintf (fp, "<body>\n");
}

int getline (char *buf) {
	char *b;
	int c;

	if (feof (stdin))
		return EOF;
	else {
		b = buf;
		while ( (c = getchar()) != '\n' && c != 12 && c != EOF ) {
			*b++ = c;
		}
		*b = '\0';
		weedout (buf);
		compact (buf);
		return (strlen (buf));
	}
}

void weedout (char *buf) {
	char *b;
	char *p;

	for (p = b = buf; *p; p++) {
		if (*p == 13) {          /* skip ^M */
			continue;
		}
		else if (*p == 27) {     /* escape character */
			if (*++p == 'E')
				continue;      /* skip \027E */
			else if (*p == '&' && *++p == 'a' && isdigit (*++p) &&
					(*++p == 'C' || (isdigit (*p) && *++p == 'C')))
				*b++ = ' ';    /* replace \027&a#C with a space */
			else
				fatal ("unrecognized escape sequence");
		}
		else {
			*b++ = *p;
		}
	}
	*b = '\0';
}

void compact (char *buf) {
	char *b;
	char *p;
	char last = 0;

	for (p = b = buf; *p; p++) {
		if (*p != ' ' || last != ' ')
			last = *b++ = *p;
	}
	*b = '\0';
}

int starts (char *s, char *pat) {
	int n;

	n = strlen (pat);
	if (!strncmp (s, pat, n)) {
		return 1;
	}
	else
		return 0;
}

char *strtrim (char *s) {
	char *p;

	if (!s)
		return NULL;
	else {
		/* find end of string */
		for (p = s; *p; p++)
			;

		/* back up to last character to keep */
		while (p != s && (!*p || isspace (*p))) {
			p--;
		}

		/* write '\0' after it */
		if (!*p || isspace (*p)) {     /* string was ALL whitespace, or empty */
			*p = '\0';
		}
		else {
			*++p = '\0';
		}

		/* find first character to keep */
		for (p = s; isspace (*p); p++)
			;
		return p;
	}
}

char *usage (void) {
	return "usage:  split_report [-images] output_dir < input_file";
}

void fatal (char *msg) {
	fprintf (stderr, "%s\n", msg);
	exit (1);
}
